@@ -73,6 +73,9 @@ use Exception::Class (%e);
 
 HTML::Mason::Exception->Trace(1);
 
+# To avoid circular reference between exception and request.
+HTML::Mason::Exception->NoRefs(1);
+
 # The import() method allows this:
 #  use HTML::Mason::Exceptions(abbr => ['error1', 'error2', ...]);
 # ...
