@@ -118,24 +118,6 @@ use HTML::Mason::MethodMaker
 			      },
       );
 
-BEGIN {
-    #
-    # We've found at least one generated component that for some
-    # reason never returns from eval-ing its object code.  For
-    # systems that provide alarm we can try to protect against
-    # this.
-    #
-    # This appears to be a Perl bug, fixed in 5.7.3+.  We'll skip
-    # this hack for versions where the bug is fixed.  To eliminate the
-    # checking penalty, we make it a compile-time constant.
-    #
-    if ( $Config{d_alarm} && $] < 5.007003 ) {
-	eval 'sub PERL_BUG_INFINITE_LOOP () { 1 }';
-    } else {
-	eval 'sub PERL_BUG_INFINITE_LOOP () { 0 }';
-    }
-}
-
 sub new
 {
     my $class = shift;
@@ -317,8 +299,12 @@ sub load {
 	# We are using object files.  Update object file if necessary
 	# and load component from there.
 	#
+	my $tries = 0;
 	do
 	{
+	    if ($tries++ == 3) {
+		$self->_compilation_error( $source->friendly_name, "Could not load or recreate object file after 3 tries" );
+	    }
 	    if ($objfilemod < $srcmod) {
 		$self->compiler->compile_to_file( file => $objfile, source => $source);
 	    }
@@ -536,18 +522,16 @@ sub eval_object_code
     my $warnstr = '';
 
     {
-	local $^W = 1 unless $ignore_expr eq '.';
+	local $^W = $ignore_expr eq '.' ? 0 : 1;
 	local $SIG{__WARN__} =
 	    ( $ignore_expr ?
-	      sub { $warnstr .= $_[0] if $_[0] !~ /$ignore_expr/ } :
-	      sub { $warnstr .= $_[0] } ) unless $ignore_expr eq '.';
-	
-	local $SIG{ALRM} = sub { die $warnstr } if PERL_BUG_INFINITE_LOOP;
-	alarm 5 if PERL_BUG_INFINITE_LOOP;
+              ( $ignore_expr eq '.' ?
+                sub { } :
+                sub { $warnstr .= $_[0] if $_[0] !~ /$ignore_expr/ }
+              ) :
+	      sub { $warnstr .= $_[0] } );
 	
 	$comp = $self->_do_or_eval(\%p);
-	
-	alarm 0 if PERL_BUG_INFINITE_LOOP;
     }
 
     $err = $warnstr . $@;
@@ -787,7 +771,7 @@ HTML::Mason::Interp - Mason Component Interpreter
 
 =head1 SYNOPSIS
 
-    my $i = new HTML::Mason::Interp (data_dir=>'/usr/local/mason',
+    my $i = HTML::Mason::Interp->new (data_dir=>'/usr/local/mason',
                                      comp_root=>'/usr/local/www/htdocs/',
                                      ...other params...);
 
@@ -931,7 +915,7 @@ All of the above properties have standard accessor methods of the same
 name. In general, no arguments retrieves the value, and one argument
 sets and returns the value.  For example:
 
-    my $interp = new HTML::Mason::Interp (...);
+    my $interp = HTML::Mason::Interp->new (...);
     my $c = $interp->compiler;
     $interp->code_cache_max_size(20 * 1024 * 1024);
 
