@@ -128,14 +128,15 @@ sub get_test_params {
 	}
     } until ($found);
     system "$Config{lns} $httpd t/httpd";
+    $conf{httpd} = $httpd;
 
     # Default: search for dynamic dependencies if mod_so is present, don't bother otherwise.
     my $default = (`t/httpd -l` =~ /mod_so\.c/ ? 'y' : 'n');
     if (lc _ask("Search existing config file for dynamic module dependencies?", $default) eq 'y') {
-	my %compiled = $pkg->_get_compilation_params('t/httpd');
+	my %compiled = $pkg->get_compilation_params('t/httpd');
 
-	my $file = _ask("  Config file", $compiled{SERVER_CONFIG_FILE}, 1);
-	$conf{modules} = $pkg->_read_existing_conf($file);
+	$conf{config_file} = _ask("  Config file", $compiled{SERVER_CONFIG_FILE}, 1);
+	$conf{modules} = $pkg->_read_existing_conf($conf{config_file});
     }
 
     # Get default user (apache doesn't like to run as root, special-case it)
@@ -150,7 +151,7 @@ sub get_test_params {
     return %conf;
 }
 
-sub _get_compilation_params {
+sub get_compilation_params {
     my ($self, $httpd) = @_;
 
     my %compiled;
@@ -183,7 +184,7 @@ sub _read_existing_conf {
 	warn "ADDED INC $file\n";
     }
 
-    my @modules       =   grep /^\s*(Add|Load)Module/, @lines;
+    my @modules       =   grep /^\s*(Add|Load|Clear)Module/, @lines;
 
     my ($server_root) = (map /^\s*ServerRoot\s*(\S+)/, @lines);
     $server_root =~ s/^"//;
@@ -258,6 +259,8 @@ sub _find_mod_perl_httpd {
     foreach ( '/usr/local/apache/bin/httpd',
 	      '/usr/local/apache_mp/bin/httpd',
 	      '/opt/apache/bin/httpd',
+	      '/usr/sbin/apache-perl',
+	      '/usr/sbin/apache',
 	      $self->_which('httpd'),
 	      $self->_which('apache'),
 	    ) {
@@ -270,7 +273,7 @@ sub _httpd_has_mod_perl {
 
     return 1 if `$httpd -l` =~ /mod_perl\.c/;
 
-    my %compiled = $self->_get_compilation_params($httpd);
+    my %compiled = $self->get_compilation_params($httpd);
 
     if ($compiled{SERVER_CONFIG_FILE}) {
 	local *SERVER_CONF;
@@ -334,24 +337,6 @@ sub simple_fetch {
     $response->is_success;
 }
 
-#even if eval $mod fails, the .pm ends up in %INC
-#so the next eval $mod succeeds, when it shouldnot
-
-my %really_have = (
-   'Apache::Table' => sub { 
-       if ($ENV{MOD_PERL}) {
-	   return Apache::Table->can('TIEHASH');
-       }
-       else {
-	   return $net::callback_hooks{PERL_TABLE_API};
-       }
-   },
-);
-
-for (qw(Apache::Cookie Apache::Request)) {
-    $really_have{$_} = $really_have{'Apache::Table'};
-}
-
 sub have_module {
     my $mod = shift;
     my $v = shift;
@@ -382,9 +367,6 @@ sub have_module {
 	warn "$@\n";
     }
 
-    if (my $cv = $really_have{$mod}) {
-	return 0 unless $cv->();
-    }
 
     print "module $mod is installed\n" unless $ENV{MOD_PERL};
     
